/* GomHashThreeMap.c generated by valac 0.44.3, the Vala compiler
 * generated from GomHashThreeMap.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * GomHashThreeMap.vala
 *
 * Copyright (C) 2016  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib-object.h>
#include <gee.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>

#define GXML_TYPE_COLLECTION (gxml_collection_get_type ())
#define GXML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_COLLECTION, GXmlCollection))
#define GXML_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_COLLECTION))
#define GXML_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_COLLECTION, GXmlCollectionIface))

typedef struct _GXmlCollection GXmlCollection;
typedef struct _GXmlCollectionIface GXmlCollectionIface;

#define GXML_TYPE_DOM_ELEMENT (gxml_dom_element_get_type ())
#define GXML_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElement))
#define GXML_IS_DOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT))
#define GXML_DOM_ELEMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_ELEMENT, GXmlDomElementIface))

typedef struct _GXmlDomElement GXmlDomElement;
typedef struct _GXmlDomElementIface GXmlDomElementIface;

#define GXML_TYPE_DOM_NODE (gxml_dom_node_get_type ())
#define GXML_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE, GXmlDomNode))
#define GXML_IS_DOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE))
#define GXML_DOM_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE, GXmlDomNodeIface))

typedef struct _GXmlDomNode GXmlDomNode;
typedef struct _GXmlDomNodeIface GXmlDomNodeIface;

#define GXML_TYPE_DOM_EVENT_TARGET (gxml_dom_event_target_get_type ())
#define GXML_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTarget))
#define GXML_IS_DOM_EVENT_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_TARGET))
#define GXML_DOM_EVENT_TARGET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_TARGET, GXmlDomEventTargetIface))

typedef struct _GXmlDomEventTarget GXmlDomEventTarget;
typedef struct _GXmlDomEventTargetIface GXmlDomEventTargetIface;

#define GXML_TYPE_DOM_EVENT_LISTENER (gxml_dom_event_listener_get_type ())
#define GXML_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListener))
#define GXML_IS_DOM_EVENT_LISTENER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT_LISTENER))
#define GXML_DOM_EVENT_LISTENER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT_LISTENER, GXmlDomEventListenerIface))

typedef struct _GXmlDomEventListener GXmlDomEventListener;
typedef struct _GXmlDomEventListenerIface GXmlDomEventListenerIface;

#define GXML_TYPE_DOM_EVENT (gxml_dom_event_get_type ())
#define GXML_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEvent))
#define GXML_IS_DOM_EVENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_EVENT))
#define GXML_DOM_EVENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_EVENT, GXmlDomEventIface))

typedef struct _GXmlDomEvent GXmlDomEvent;
typedef struct _GXmlDomEventIface GXmlDomEventIface;

#define GXML_TYPE_DOM_TIME_STAMP (gxml_dom_time_stamp_get_type ())
#define GXML_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStamp))
#define GXML_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))
#define GXML_IS_DOM_TIME_STAMP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_IS_DOM_TIME_STAMP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_TIME_STAMP))
#define GXML_DOM_TIME_STAMP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_TIME_STAMP, GXmlDomTimeStampClass))

typedef struct _GXmlDomTimeStamp GXmlDomTimeStamp;
typedef struct _GXmlDomTimeStampClass GXmlDomTimeStampClass;
typedef enum  {
	GXML_DOM_EVENT_PHASE_NONE = 0,
	GXML_DOM_EVENT_PHASE_CAPTURING_PHASE,
	GXML_DOM_EVENT_PHASE_AT_TARGET,
	GXML_DOM_EVENT_PHASE_BUBBLING_PHASE
} GXmlDomEventPhase;

#define GXML_DOM_EVENT_TYPE_PHASE (gxml_dom_event_phase_get_type ())
typedef enum  {
	GXML_DOM_NODE_DOCUMENT_POSITION_NONE = 1 << 0,
	GXML_DOM_NODE_DOCUMENT_POSITION_DISCONNECTED = 1 << 1,
	GXML_DOM_NODE_DOCUMENT_POSITION_PRECEDING = 1 << 2,
	GXML_DOM_NODE_DOCUMENT_POSITION_FOLLOWING = 1 << 3,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINS = 1 << 4,
	GXML_DOM_NODE_DOCUMENT_POSITION_CONTAINED_BY = 1 << 5,
	GXML_DOM_NODE_DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC = 1 << 6
} GXmlDomNodeDocumentPosition;

#define GXML_DOM_NODE_TYPE_DOCUMENT_POSITION (gxml_dom_node_document_position_get_type ())
typedef enum  {
	GXML_DOM_NODE_NODE_TYPE_INVALID = 0,
	GXML_DOM_NODE_NODE_TYPE_ELEMENT_NODE = 1,
	GXML_DOM_NODE_NODE_TYPE_ATTRIBUTE_NODE,
	GXML_DOM_NODE_NODE_TYPE_TEXT_NODE,
	GXML_DOM_NODE_NODE_TYPE_CDATA_SECTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_REFERENCE_NODE,
	GXML_DOM_NODE_NODE_TYPE_ENTITY_NODE,
	GXML_DOM_NODE_NODE_TYPE_PROCESSING_INSTRUCTION_NODE,
	GXML_DOM_NODE_NODE_TYPE_COMMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_TYPE_NODE,
	GXML_DOM_NODE_NODE_TYPE_DOCUMENT_FRAGMENT_NODE,
	GXML_DOM_NODE_NODE_TYPE_NOTATION_NODE
} GXmlDomNodeNodeType;

#define GXML_DOM_NODE_TYPE_NODE_TYPE (gxml_dom_node_node_type_get_type ())

#define GXML_TYPE_DOM_DOCUMENT (gxml_dom_document_get_type ())
#define GXML_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocument))
#define GXML_IS_DOM_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT))
#define GXML_DOM_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT, GXmlDomDocumentIface))

typedef struct _GXmlDomDocument GXmlDomDocument;
typedef struct _GXmlDomDocumentIface GXmlDomDocumentIface;

#define GXML_TYPE_DOM_PARENT_NODE (gxml_dom_parent_node_get_type ())
#define GXML_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNode))
#define GXML_IS_DOM_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PARENT_NODE))
#define GXML_DOM_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PARENT_NODE, GXmlDomParentNodeIface))

typedef struct _GXmlDomParentNode GXmlDomParentNode;
typedef struct _GXmlDomParentNodeIface GXmlDomParentNodeIface;

#define GXML_TYPE_DOM_NODE_LIST (gxml_dom_node_list_get_type ())
#define GXML_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeList))
#define GXML_IS_DOM_NODE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_LIST))
#define GXML_DOM_NODE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_LIST, GXmlDomNodeListIface))

typedef struct _GXmlDomNodeList GXmlDomNodeList;
typedef struct _GXmlDomNodeListIface GXmlDomNodeListIface;

#define GXML_TYPE_DOM_ELEMENT_LIST (gxml_dom_element_list_get_type ())
#define GXML_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementList))
#define GXML_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))
#define GXML_IS_DOM_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_IS_DOM_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_ELEMENT_LIST))
#define GXML_DOM_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_ELEMENT_LIST, GXmlDomElementListClass))

typedef struct _GXmlDomElementList GXmlDomElementList;
typedef struct _GXmlDomElementListClass GXmlDomElementListClass;

#define GXML_TYPE_DOM_HTML_COLLECTION (gxml_dom_html_collection_get_type ())
#define GXML_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollection))
#define GXML_IS_DOM_HTML_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_HTML_COLLECTION))
#define GXML_DOM_HTML_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_HTML_COLLECTION, GXmlDomHTMLCollectionIface))

typedef struct _GXmlDomHTMLCollection GXmlDomHTMLCollection;
typedef struct _GXmlDomHTMLCollectionIface GXmlDomHTMLCollectionIface;

#define GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE (gxml_dom_non_element_parent_node_get_type ())
#define GXML_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNode))
#define GXML_IS_DOM_NON_ELEMENT_PARENT_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE))
#define GXML_DOM_NON_ELEMENT_PARENT_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_ELEMENT_PARENT_NODE, GXmlDomNonElementParentNodeIface))

typedef struct _GXmlDomNonElementParentNode GXmlDomNonElementParentNode;
typedef struct _GXmlDomNonElementParentNodeIface GXmlDomNonElementParentNodeIface;

#define GXML_TYPE_DOM_DOCUMENT_FRAGMENT (gxml_dom_document_fragment_get_type ())
#define GXML_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragment))
#define GXML_IS_DOM_DOCUMENT_FRAGMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT))
#define GXML_DOM_DOCUMENT_FRAGMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_FRAGMENT, GXmlDomDocumentFragmentIface))

typedef struct _GXmlDomDocumentFragment GXmlDomDocumentFragment;
typedef struct _GXmlDomDocumentFragmentIface GXmlDomDocumentFragmentIface;

#define GXML_TYPE_DOM_TEXT (gxml_dom_text_get_type ())
#define GXML_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TEXT, GXmlDomText))
#define GXML_IS_DOM_TEXT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TEXT))
#define GXML_DOM_TEXT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TEXT, GXmlDomTextIface))

typedef struct _GXmlDomText GXmlDomText;
typedef struct _GXmlDomTextIface GXmlDomTextIface;

#define GXML_TYPE_DOM_CHARACTER_DATA (gxml_dom_character_data_get_type ())
#define GXML_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterData))
#define GXML_IS_DOM_CHARACTER_DATA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHARACTER_DATA))
#define GXML_DOM_CHARACTER_DATA_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHARACTER_DATA, GXmlDomCharacterDataIface))

typedef struct _GXmlDomCharacterData GXmlDomCharacterData;
typedef struct _GXmlDomCharacterDataIface GXmlDomCharacterDataIface;

#define GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE (gxml_dom_non_document_type_child_node_get_type ())
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNode))
#define GXML_IS_DOM_NON_DOCUMENT_TYPE_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE))
#define GXML_DOM_NON_DOCUMENT_TYPE_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NON_DOCUMENT_TYPE_CHILD_NODE, GXmlDomNonDocumentTypeChildNodeIface))

typedef struct _GXmlDomNonDocumentTypeChildNode GXmlDomNonDocumentTypeChildNode;
typedef struct _GXmlDomNonDocumentTypeChildNodeIface GXmlDomNonDocumentTypeChildNodeIface;

#define GXML_TYPE_DOM_CHILD_NODE (gxml_dom_child_node_get_type ())
#define GXML_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNode))
#define GXML_IS_DOM_CHILD_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_CHILD_NODE))
#define GXML_DOM_CHILD_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_CHILD_NODE, GXmlDomChildNodeIface))

typedef struct _GXmlDomChildNode GXmlDomChildNode;
typedef struct _GXmlDomChildNodeIface GXmlDomChildNodeIface;

#define GXML_TYPE_DOM_COMMENT (gxml_dom_comment_get_type ())
#define GXML_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomComment))
#define GXML_IS_DOM_COMMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_COMMENT))
#define GXML_DOM_COMMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_COMMENT, GXmlDomCommentIface))

typedef struct _GXmlDomComment GXmlDomComment;
typedef struct _GXmlDomCommentIface GXmlDomCommentIface;

#define GXML_TYPE_DOM_PROCESSING_INSTRUCTION (gxml_dom_processing_instruction_get_type ())
#define GXML_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstruction))
#define GXML_IS_DOM_PROCESSING_INSTRUCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION))
#define GXML_DOM_PROCESSING_INSTRUCTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_PROCESSING_INSTRUCTION, GXmlDomProcessingInstructionIface))

typedef struct _GXmlDomProcessingInstruction GXmlDomProcessingInstruction;
typedef struct _GXmlDomProcessingInstructionIface GXmlDomProcessingInstructionIface;

#define GXML_TYPE_DOM_RANGE (gxml_dom_range_get_type ())
#define GXML_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRange))
#define GXML_IS_DOM_RANGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_RANGE))
#define GXML_DOM_RANGE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_RANGE, GXmlDomRangeIface))

typedef struct _GXmlDomRange GXmlDomRange;
typedef struct _GXmlDomRangeIface GXmlDomRangeIface;
typedef enum  {
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_START = 0,
	GXML_DOM_RANGE_BOUNDARY_POINTS_START_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_END,
	GXML_DOM_RANGE_BOUNDARY_POINTS_END_TO_START
} GXmlDomRangeBoundaryPoints;

#define GXML_DOM_RANGE_TYPE_BOUNDARY_POINTS (gxml_dom_range_boundary_points_get_type ())

#define GXML_TYPE_DOM_NODE_FILTER (gxml_dom_node_filter_get_type ())
#define GXML_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilter))
#define GXML_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))
#define GXML_IS_DOM_NODE_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_IS_DOM_NODE_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_DOM_NODE_FILTER))
#define GXML_DOM_NODE_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_DOM_NODE_FILTER, GXmlDomNodeFilterClass))

typedef struct _GXmlDomNodeFilter GXmlDomNodeFilter;
typedef struct _GXmlDomNodeFilterClass GXmlDomNodeFilterClass;

#define GXML_TYPE_DOM_NODE_ITERATOR (gxml_dom_node_iterator_get_type ())
#define GXML_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIterator))
#define GXML_IS_DOM_NODE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NODE_ITERATOR))
#define GXML_DOM_NODE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NODE_ITERATOR, GXmlDomNodeIteratorIface))

typedef struct _GXmlDomNodeIterator GXmlDomNodeIterator;
typedef struct _GXmlDomNodeIteratorIface GXmlDomNodeIteratorIface;

#define GXML_TYPE_DOM_TREE_WALKER (gxml_dom_tree_walker_get_type ())
#define GXML_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalker))
#define GXML_IS_DOM_TREE_WALKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TREE_WALKER))
#define GXML_DOM_TREE_WALKER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TREE_WALKER, GXmlDomTreeWalkerIface))

typedef struct _GXmlDomTreeWalker GXmlDomTreeWalker;
typedef struct _GXmlDomTreeWalkerIface GXmlDomTreeWalkerIface;

#define GXML_TYPE_PARSER (gxml_parser_get_type ())
#define GXML_PARSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_PARSER, GXmlParser))
#define GXML_IS_PARSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_PARSER))
#define GXML_PARSER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_PARSER, GXmlParserIface))

typedef struct _GXmlParser GXmlParser;
typedef struct _GXmlParserIface GXmlParserIface;

#define GXML_TYPE_DOM_IMPLEMENTATION (gxml_dom_implementation_get_type ())
#define GXML_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementation))
#define GXML_IS_DOM_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_IMPLEMENTATION))
#define GXML_DOM_IMPLEMENTATION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_IMPLEMENTATION, GXmlDomImplementationIface))

typedef struct _GXmlDomImplementation GXmlDomImplementation;
typedef struct _GXmlDomImplementationIface GXmlDomImplementationIface;

#define GXML_TYPE_DOM_DOCUMENT_TYPE (gxml_dom_document_type_get_type ())
#define GXML_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentType))
#define GXML_IS_DOM_DOCUMENT_TYPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE))
#define GXML_DOM_DOCUMENT_TYPE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_DOCUMENT_TYPE, GXmlDomDocumentTypeIface))

typedef struct _GXmlDomDocumentType GXmlDomDocumentType;
typedef struct _GXmlDomDocumentTypeIface GXmlDomDocumentTypeIface;

#define GXML_TYPE_DOM_XML_DOCUMENT (gxml_dom_xml_document_get_type ())
#define GXML_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocument))
#define GXML_IS_DOM_XML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_XML_DOCUMENT))
#define GXML_DOM_XML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_XML_DOCUMENT, GXmlDomXMLDocumentIface))

typedef struct _GXmlDomXMLDocument GXmlDomXMLDocument;
typedef struct _GXmlDomXMLDocumentIface GXmlDomXMLDocumentIface;

#define GXML_TYPE_DOCUMENT (gxml_document_get_type ())
#define GXML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT, GXmlDocument))
#define GXML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT))
#define GXML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT, GXmlDocumentIface))

typedef struct _GXmlDocument GXmlDocument;
typedef struct _GXmlDocumentIface GXmlDocumentIface;

#define GXML_TYPE_NODE (gxml_node_get_type ())
#define GXML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE, GXmlNode))
#define GXML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE))
#define GXML_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NODE, GXmlNodeIface))

typedef struct _GXmlNode GXmlNode;
typedef struct _GXmlNodeIface GXmlNodeIface;

#define GXML_TYPE_ELEMENT_LIST (gxml_element_list_get_type ())
#define GXML_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementList))
#define GXML_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))
#define GXML_IS_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT_LIST))
#define GXML_IS_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ELEMENT_LIST))
#define GXML_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))

typedef struct _GXmlElementList GXmlElementList;
typedef struct _GXmlElementListClass GXmlElementListClass;

#define GXML_TYPE_NAMESPACE (gxml_namespace_get_type ())
#define GXML_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NAMESPACE, GXmlNamespace))
#define GXML_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NAMESPACE))
#define GXML_NAMESPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NAMESPACE, GXmlNamespaceIface))

typedef struct _GXmlNamespace GXmlNamespace;
typedef struct _GXmlNamespaceIface GXmlNamespaceIface;
typedef enum  {
	GXML_NODE_TYPE_INVALID = 0,
	GXML_NODE_TYPE_ELEMENT = 1,
	GXML_NODE_TYPE_ATTRIBUTE,
	GXML_NODE_TYPE_TEXT,
	GXML_NODE_TYPE_CDATA_SECTION,
	GXML_NODE_TYPE_ENTITY_REFERENCE,
	GXML_NODE_TYPE_ENTITY,
	GXML_NODE_TYPE_PROCESSING_INSTRUCTION,
	GXML_NODE_TYPE_COMMENT,
	GXML_NODE_TYPE_DOCUMENT,
	GXML_NODE_TYPE_DOCUMENT_TYPE,
	GXML_NODE_TYPE_DOCUMENT_FRAGMENT,
	GXML_NODE_TYPE_NOTATION
} GXmlNodeType;

#define GXML_TYPE_NODE_TYPE (gxml_node_type_get_type ())

#define GXML_TYPE_DOM_TOKEN_LIST (gxml_dom_token_list_get_type ())
#define GXML_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenList))
#define GXML_IS_DOM_TOKEN_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_TOKEN_LIST))
#define GXML_DOM_TOKEN_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_TOKEN_LIST, GXmlDomTokenListIface))

typedef struct _GXmlDomTokenList GXmlDomTokenList;
typedef struct _GXmlDomTokenListIface GXmlDomTokenListIface;

#define GXML_TYPE_DOM_NAMED_NODE_MAP (gxml_dom_named_node_map_get_type ())
#define GXML_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMap))
#define GXML_IS_DOM_NAMED_NODE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP))
#define GXML_DOM_NAMED_NODE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOM_NAMED_NODE_MAP, GXmlDomNamedNodeMapIface))

typedef struct _GXmlDomNamedNodeMap GXmlDomNamedNodeMap;
typedef struct _GXmlDomNamedNodeMapIface GXmlDomNamedNodeMapIface;

#define GXML_TYPE_BASE_COLLECTION (gxml_base_collection_get_type ())
#define GXML_BASE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollection))
#define GXML_BASE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollectionClass))
#define GXML_IS_BASE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_BASE_COLLECTION))
#define GXML_IS_BASE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_BASE_COLLECTION))
#define GXML_BASE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_BASE_COLLECTION, GXmlBaseCollectionClass))

typedef struct _GXmlBaseCollection GXmlBaseCollection;
typedef struct _GXmlBaseCollectionClass GXmlBaseCollectionClass;
typedef struct _GXmlBaseCollectionPrivate GXmlBaseCollectionPrivate;

#define GXML_TYPE_GOM_NODE (gxml_gom_node_get_type ())
#define GXML_GOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_NODE, GXmlGomNode))
#define GXML_GOM_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_NODE, GXmlGomNodeClass))
#define GXML_IS_GOM_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_NODE))
#define GXML_IS_GOM_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_NODE))
#define GXML_GOM_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_NODE, GXmlGomNodeClass))

typedef struct _GXmlGomNode GXmlGomNode;
typedef struct _GXmlGomNodeClass GXmlGomNodeClass;

#define GXML_TYPE_GOM_ELEMENT (gxml_gom_element_get_type ())
#define GXML_GOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_ELEMENT, GXmlGomElement))
#define GXML_GOM_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_ELEMENT, GXmlGomElementClass))
#define GXML_IS_GOM_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_ELEMENT))
#define GXML_IS_GOM_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_ELEMENT))
#define GXML_GOM_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_ELEMENT, GXmlGomElementClass))

typedef struct _GXmlGomElement GXmlGomElement;
typedef struct _GXmlGomElementClass GXmlGomElementClass;

#define GXML_TYPE_THREE_MAP (gxml_three_map_get_type ())
#define GXML_THREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_THREE_MAP, GXmlThreeMap))
#define GXML_IS_THREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_THREE_MAP))
#define GXML_THREE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_THREE_MAP, GXmlThreeMapIface))

typedef struct _GXmlThreeMap GXmlThreeMap;
typedef struct _GXmlThreeMapIface GXmlThreeMapIface;

#define GXML_TYPE_GOM_HASH_THREE_MAP (gxml_gom_hash_three_map_get_type ())
#define GXML_GOM_HASH_THREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMap))
#define GXML_GOM_HASH_THREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMapClass))
#define GXML_IS_GOM_HASH_THREE_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_GOM_HASH_THREE_MAP))
#define GXML_IS_GOM_HASH_THREE_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_GOM_HASH_THREE_MAP))
#define GXML_GOM_HASH_THREE_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMapClass))

typedef struct _GXmlGomHashThreeMap GXmlGomHashThreeMap;
typedef struct _GXmlGomHashThreeMapClass GXmlGomHashThreeMapClass;
typedef struct _GXmlGomHashThreeMapPrivate GXmlGomHashThreeMapPrivate;
enum  {
	GXML_GOM_HASH_THREE_MAP_0_PROPERTY,
	GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY,
	GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY,
	GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY,
	GXML_GOM_HASH_THREE_MAP_PRIMARY_KEYS_SET_PROPERTY,
	GXML_GOM_HASH_THREE_MAP_NUM_PROPERTIES
};
static GParamSpec* gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

#define GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY (gxml_mappeable_element_three_key_get_type ())
#define GXML_MAPPEABLE_ELEMENT_THREE_KEY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY, GXmlMappeableElementThreeKey))
#define GXML_IS_MAPPEABLE_ELEMENT_THREE_KEY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY))
#define GXML_MAPPEABLE_ELEMENT_THREE_KEY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY, GXmlMappeableElementThreeKeyIface))

typedef struct _GXmlMappeableElementThreeKey GXmlMappeableElementThreeKey;
typedef struct _GXmlMappeableElementThreeKeyIface GXmlMappeableElementThreeKeyIface;

struct _GXmlDomEventIface {
	GTypeInterface parent_iface;
	void (*stop_propagation) (GXmlDomEvent* self);
	void (*stop_immediate_propagation) (GXmlDomEvent* self);
	void (*prevent_default) (GXmlDomEvent* self);
	void (*init_event) (GXmlDomEvent* self, const gchar* type, gboolean bubbles, gboolean cancelable);
	const gchar* (*get_etype) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_event_target) (GXmlDomEvent* self);
	GXmlDomEventTarget* (*get_current_target) (GXmlDomEvent* self);
	gboolean (*get_bubbles) (GXmlDomEvent* self);
	gboolean (*get_cancelable) (GXmlDomEvent* self);
	gboolean (*get_is_trusted) (GXmlDomEvent* self);
	GXmlDomTimeStamp* (*get_time_stamp) (GXmlDomEvent* self);
	gboolean (*get_default_prevented) (GXmlDomEvent* self);
	GXmlDomEventPhase (*get_event_phase) (GXmlDomEvent* self);
};

struct _GXmlDomEventListenerIface {
	GTypeInterface parent_iface;
	void (*handle_event) (GXmlDomEventListener* self, GXmlDomEvent* event);
};

struct _GXmlDomEventTargetIface {
	GTypeInterface parent_iface;
	void (*add_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	void (*remove_event_listener) (GXmlDomEventTarget* self, const gchar* type, GXmlDomEventListener* callback, gboolean capture);
	gboolean (*dispatch_event) (GXmlDomEventTarget* self, GXmlDomEvent* event, GError** error);
};

struct _GXmlDomNodeListIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNodeList* self, gint index);
	gint (*get_length) (GXmlDomNodeList* self);
};

struct _GXmlDomHTMLCollectionIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement** (*to_array) (GXmlDomHTMLCollection* self, gint* result_length1);
	GXmlDomElement* (*item) (GXmlDomHTMLCollection* self, gint index);
	GXmlDomElement* (*named_item) (GXmlDomHTMLCollection* self, const gchar* name);
	gint (*get_length) (GXmlDomHTMLCollection* self);
};

struct _GXmlDomParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*query_selector) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomNodeList* (*query_selector_all) (GXmlDomParentNode* self, const gchar* selectors, GError** error);
	GXmlDomElementList* (*get_elements_by_property_value) (GXmlDomParentNode* self, const gchar* property, const gchar* value);
	GXmlDomHTMLCollection* (*get_children) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_first_element_child) (GXmlDomParentNode* self);
	GXmlDomElement* (*get_last_element_child) (GXmlDomParentNode* self);
	gint (*get_child_element_count) (GXmlDomParentNode* self);
};

struct _GXmlDomNonElementParentNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_element_by_id) (GXmlDomNonElementParentNode* self, const gchar* element_id, GError** error);
};

struct _GXmlDomDocumentFragmentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomNonDocumentTypeChildNodeIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*get_previous_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
	GXmlDomElement* (*get_next_element_sibling) (GXmlDomNonDocumentTypeChildNode* self);
};

struct _GXmlDomChildNodeIface {
	GTypeInterface parent_iface;
	void (*remove) (GXmlDomChildNode* self);
};

struct _GXmlDomCharacterDataIface {
	GTypeInterface parent_iface;
	gchar* (*substring_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*append_data) (GXmlDomCharacterData* self, const gchar* data);
	void (*insert_data) (GXmlDomCharacterData* self, gint offset, const gchar* data, GError** error);
	void (*delete_data) (GXmlDomCharacterData* self, gint offset, gint count, GError** error);
	void (*replace_data) (GXmlDomCharacterData* self, gint offset, gint count, const gchar* data, GError** error);
	gchar* (*get_data) (GXmlDomCharacterData* self);
	void (*set_data) (GXmlDomCharacterData* self, const gchar* value);
	gint (*get_length) (GXmlDomCharacterData* self);
};

struct _GXmlDomTextIface {
	GTypeInterface parent_iface;
	GXmlDomText* (*split_text) (GXmlDomText* self, gint offset, GError** error);
	gchar* (*get_whole_text) (GXmlDomText* self);
};

struct _GXmlDomCommentIface {
	GTypeInterface parent_iface;
};

struct _GXmlDomProcessingInstructionIface {
	GTypeInterface parent_iface;
	gchar* (*get_target) (GXmlDomProcessingInstruction* self);
};

struct _GXmlDomRangeIface {
	GTypeInterface parent_iface;
	void (*set_start) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_end) (GXmlDomRange* self, GXmlDomNode* node, gint offset, GError** error);
	void (*set_start_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_start_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_before) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*set_end_after) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*collapse) (GXmlDomRange* self, gboolean to_start, GError** error);
	void (*select_node) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	void (*select_node_contents) (GXmlDomRange* self, GXmlDomNode* node, GError** error);
	gint (*compare_boundary_points) (GXmlDomRange* self, GXmlDomRangeBoundaryPoints how, GXmlDomRange* sourceRange, GError** error);
	void (*delete_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*extract_contents) (GXmlDomRange* self, GError** error);
	GXmlDomDocumentFragment* (*clone_contents) (GXmlDomRange* self, GError** error);
	void (*insert_node) (GXmlDomRange* self, GXmlDomNode* node);
	void (*surround_contents) (GXmlDomRange* self, GXmlDomNode* newParent);
	GXmlDomRange* (*clone_range) (GXmlDomRange* self);
	void (*detach) (GXmlDomRange* self);
	gboolean (*is_point_in_range) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gshort (*compare_point) (GXmlDomRange* self, GXmlDomNode* node, gint offset);
	gboolean (*intersects_node) (GXmlDomRange* self, GXmlDomNode* node);
	gchar* (*to_string) (GXmlDomRange* self);
	GXmlDomNode* (*get_start_container) (GXmlDomRange* self);
	gint (*get_start_offset) (GXmlDomRange* self);
	GXmlDomNode* (*get_end_container) (GXmlDomRange* self);
	gint (*get_end_offset) (GXmlDomRange* self);
	gboolean (*get_collapsed) (GXmlDomRange* self);
	GXmlDomNode* (*get_common_ancestor_container) (GXmlDomRange* self);
};

struct _GXmlDomNodeIteratorIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*next_node) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*previous_node) (GXmlDomNodeIterator* self);
	void (*detach) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_root) (GXmlDomNodeIterator* self);
	GXmlDomNode* (*get_reference_node) (GXmlDomNodeIterator* self);
	gboolean (*get_pointer_before_reference_node) (GXmlDomNodeIterator* self);
	gint (*get_what_to_show) (GXmlDomNodeIterator* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomNodeIterator* self);
};

struct _GXmlDomTreeWalkerIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*parent_node) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*first_child) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*last_child) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previous_sibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*next_sibling) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*previous_node) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*next_node) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_root) (GXmlDomTreeWalker* self);
	gint (*get_what_to_show) (GXmlDomTreeWalker* self);
	GXmlDomNodeFilter* (*get_filter) (GXmlDomTreeWalker* self);
	GXmlDomNode* (*get_current_node) (GXmlDomTreeWalker* self);
};

struct _GXmlParserIface {
	GTypeInterface parent_iface;
	void (*write_file) (GXmlParser* self, GFile* file, GError** error);
	void (*write_file_async) (GXmlParser* self, GFile* file, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_file_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	gchar* (*write_string) (GXmlParser* self, GError** error);
	void (*write_string_async) (GXmlParser* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar* (*write_string_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	void (*write_stream) (GXmlParser* self, GOutputStream* stream, GError** error);
	void (*write_stream_async) (GXmlParser* self, GOutputStream* stream, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_stream_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	void (*read_file) (GXmlParser* self, GFile* file, GError** error);
	void (*read_file_async) (GXmlParser* self, GFile* file, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_file_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	void (*read_stream) (GXmlParser* self, GInputStream* stream, GError** error);
	void (*read_stream_async) (GXmlParser* self, GInputStream* stream, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_stream_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	void (*read_string) (GXmlParser* self, const gchar* str, GError** error);
	void (*read_string_async) (GXmlParser* self, const gchar* str, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_string_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	GInputStream* (*create_stream) (GXmlParser* self, GError** error);
	void (*create_stream_async) (GXmlParser* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GInputStream* (*create_stream_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	void (*read_child_nodes) (GXmlParser* self, GXmlDomNode* parent, GError** error);
	void (*read_child_nodes_async) (GXmlParser* self, GXmlDomNode* parent, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_child_nodes_finish) (GXmlParser* self, GAsyncResult* _res_, GError** error);
	gboolean (*read_child_node) (GXmlParser* self, GXmlDomNode* parent, GError** error);
	gboolean (*read_child_element) (GXmlParser* self, GXmlDomNode* parent, GError** error);
	gboolean (*read_element_property) (GXmlParser* self, GXmlDomNode* parent, GXmlDomNode* * element, GError** error);
	gboolean (*add_element_collection) (GXmlParser* self, GXmlDomNode* parent, GXmlDomNode* * element, GError** error);
	void (*read_child_nodes_stream) (GXmlParser* self, GInputStream* istream, GError** error);
	void (*read_child_nodes_string) (GXmlParser* self, const gchar* str, GError** error);
	gchar* (*read_unparsed) (GXmlParser* self, GError** error);
	gboolean (*move_next_node) (GXmlParser* self, GError** error);
	gboolean (*current_is_empty_element) (GXmlParser* self);
	gboolean (*current_is_element) (GXmlParser* self);
	gboolean (*current_is_document) (GXmlParser* self);
	gchar* (*current_node_name) (GXmlParser* self);
	GXmlDomElement* (*create_element) (GXmlParser* self, GXmlDomNode* parent, GError** error);
	void (*read_element) (GXmlParser* self, GXmlDomElement* element, GError** error);
	gboolean (*get_backup) (GXmlParser* self);
	void (*set_backup) (GXmlParser* self, gboolean value);
	gboolean (*get_indent) (GXmlParser* self);
	void (*set_indent) (GXmlParser* self, gboolean value);
	GCancellable* (*get_cancellable) (GXmlParser* self);
	void (*set_cancellable) (GXmlParser* self, GCancellable* value);
	GXmlDomNode* (*get_node) (GXmlParser* self);
};

struct _GXmlDomDocumentTypeIface {
	GTypeInterface parent_iface;
	const gchar* (*get_name) (GXmlDomDocumentType* self);
	const gchar* (*get_public_id) (GXmlDomDocumentType* self);
	const gchar* (*get_system_id) (GXmlDomDocumentType* self);
};

struct _GXmlDomXMLDocumentIface {
	GTypeInterface parent_iface;
};

struct _GXmlNamespaceIface {
	GTypeInterface parent_iface;
	gchar* (*get_uri) (GXmlNamespace* self);
	gchar* (*get_prefix) (GXmlNamespace* self);
};

struct _GXmlNodeIface {
	GTypeInterface parent_iface;
	GXmlNode* (*get) (GXmlNode* self, const gchar* key);
	GXmlElementList* (*get_elements_by_property_value) (GXmlNode* self, const gchar* property, const gchar* value);
	GXmlElementList* (*get_elements_by_name) (GXmlNode* self, const gchar* name);
	GXmlElementList* (*get_elements_by_name_ns) (GXmlNode* self, const gchar* name, const gchar* ns);
	gchar* (*to_string) (GXmlNode* self);
	gboolean (*set_namespace) (GXmlNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*ns_prefix) (GXmlNode* self);
	gchar* (*ns_uri) (GXmlNode* self);
	GeeList* (*get_namespaces) (GXmlNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlNode* self);
	GeeMap* (*get_attrs) (GXmlNode* self);
	gchar* (*get_name) (GXmlNode* self);
	gchar* (*get_value) (GXmlNode* self);
	void (*set_value) (GXmlNode* self, const gchar* value);
	GXmlNodeType (*get_type_node) (GXmlNode* self);
	GXmlDocument* (*get_document) (GXmlNode* self);
	GXmlNode* (*get_parent) (GXmlNode* self);
};

struct _GXmlDocumentIface {
	GTypeInterface parent_iface;
	GXmlNode* (*create_element) (GXmlDocument* self, const gchar* name, GError** error);
	GXmlNode* (*create_text) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_comment) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_cdata) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_pi) (GXmlDocument* self, const gchar* target, const gchar* data);
	gboolean (*save) (GXmlDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
	gboolean (*get_indent) (GXmlDocument* self);
	void (*set_indent) (GXmlDocument* self, gboolean value);
	gboolean (*get_ns_top) (GXmlDocument* self);
	void (*set_ns_top) (GXmlDocument* self, gboolean value);
	gboolean (*get_prefix_default_ns) (GXmlDocument* self);
	void (*set_prefix_default_ns) (GXmlDocument* self, gboolean value);
	gboolean (*get_backup) (GXmlDocument* self);
	void (*set_backup) (GXmlDocument* self, gboolean value);
	GXmlNode* (*get_root) (GXmlDocument* self);
	GFile* (*get_file) (GXmlDocument* self);
	void (*set_file) (GXmlDocument* self, GFile* value);
};

struct _GXmlDomImplementationIface {
	GTypeInterface parent_iface;
	GXmlDomDocumentType* (*create_document_type) (GXmlDomImplementation* self, const gchar* qualified_name, const gchar* public_id, const gchar* system_id, GError** error);
	GXmlDomXMLDocument* (*create_document) (GXmlDomImplementation* self, const gchar* nspace, const gchar* qualified_name, GXmlDomDocumentType* doctype, GError** error);
	GXmlDocument* (*create_html_document) (GXmlDomImplementation* self, const gchar* title);
	gboolean (*has_feature) (GXmlDomImplementation* self);
};

struct _GXmlDomDocumentIface {
	GTypeInterface parent_iface;
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomDocument* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomDocument* self, const gchar* classNames);
	GXmlDomElement* (*create_element) (GXmlDomDocument* self, const gchar* local_name, GError** error);
	GXmlDomElement* (*create_element_ns) (GXmlDomDocument* self, const gchar* namespace, const gchar* qualified_name, GError** error);
	GXmlDomDocumentFragment* (*create_document_fragment) (GXmlDomDocument* self);
	GXmlDomText* (*create_text_node) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomComment* (*create_comment) (GXmlDomDocument* self, const gchar* data, GError** error);
	GXmlDomProcessingInstruction* (*create_processing_instruction) (GXmlDomDocument* self, const gchar* target, const gchar* data, GError** error);
	GXmlDomNode* (*import_node) (GXmlDomDocument* self, GXmlDomNode* node, gboolean deep, GError** error);
	GXmlDomNode* (*adopt_node) (GXmlDomDocument* self, GXmlDomNode* node, GError** error);
	GXmlDomEvent* (*create_event) (GXmlDomDocument* self, const gchar* interface, GError** error);
	GXmlDomRange* (*create_range) (GXmlDomDocument* self);
	GXmlDomNodeIterator* (*create_node_iterator) (GXmlDomDocument* self, GXmlDomNode* root, gint whatToShow, GXmlDomNodeFilter* filter);
	GXmlDomTreeWalker* (*create_tree_walker) (GXmlDomDocument* self, GXmlDomNode* root, gint what_to_show, GXmlDomNodeFilter* filter);
	void (*write_file) (GXmlDomDocument* self, GFile* file, GCancellable* cancellable, GError** error);
	void (*write_file_async) (GXmlDomDocument* self, GFile* file, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_file_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	void (*write_stream) (GXmlDomDocument* self, GOutputStream* stream, GCancellable* cancellable, GError** error);
	void (*write_stream_async) (GXmlDomDocument* self, GOutputStream* stream, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_stream_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	GInputStream* (*create_stream) (GXmlDomDocument* self, GError** error);
	void (*create_stream_async) (GXmlDomDocument* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GInputStream* (*create_stream_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	gchar* (*write_string) (GXmlDomDocument* self, GCancellable* cancellable, GError** error);
	void (*write_string_async) (GXmlDomDocument* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar* (*write_string_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	void (*read_from_file) (GXmlDomDocument* self, GFile* file, GCancellable* cancellable, GError** error);
	void (*read_from_file_async) (GXmlDomDocument* self, GFile* file, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_file_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	void (*read_from_string) (GXmlDomDocument* self, const gchar* str, GCancellable* cancellable, GError** error);
	void (*read_from_string_async) (GXmlDomDocument* self, const gchar* str, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_string_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	void (*read_from_stream) (GXmlDomDocument* self, GInputStream* stream, GCancellable* cancellable, GError** error);
	void (*read_from_stream_async) (GXmlDomDocument* self, GInputStream* stream, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_stream_finish) (GXmlDomDocument* self, GAsyncResult* _res_, GError** error);
	GXmlParser* (*get_xml_parser) (GXmlDomDocument* self);
	void (*set_xml_parser) (GXmlDomDocument* self, GXmlParser* parser);
	GXmlDomImplementation* (*get_implementation) (GXmlDomDocument* self);
	const gchar* (*get_url) (GXmlDomDocument* self);
	const gchar* (*get_document_uri) (GXmlDomDocument* self);
	const gchar* (*get_origin) (GXmlDomDocument* self);
	const gchar* (*get_compat_mode) (GXmlDomDocument* self);
	const gchar* (*get_character_set) (GXmlDomDocument* self);
	const gchar* (*get_content_type) (GXmlDomDocument* self);
	GXmlDomDocumentType* (*get_doctype) (GXmlDomDocument* self);
	GXmlDomElement* (*get_document_element) (GXmlDomDocument* self);
};

struct _GXmlDomNodeIface {
	GTypeInterface parent_iface;
	gboolean (*has_child_nodes) (GXmlDomNode* self);
	void (*normalize) (GXmlDomNode* self);
	gboolean (*is_equal_node) (GXmlDomNode* self, GXmlDomNode* node);
	GXmlDomNodeDocumentPosition (*compare_document_position) (GXmlDomNode* self, GXmlDomNode* other);
	gboolean (*contains) (GXmlDomNode* self, GXmlDomNode* other);
	gchar* (*lookup_prefix) (GXmlDomNode* self, const gchar* nspace);
	gchar* (*lookup_namespace_uri) (GXmlDomNode* self, const gchar* prefix);
	gboolean (*is_default_namespace) (GXmlDomNode* self, const gchar* nspace);
	GXmlDomNode* (*insert_before) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*append_child) (GXmlDomNode* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*replace_child) (GXmlDomNode* self, GXmlDomNode* node, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*remove_child) (GXmlDomNode* self, GXmlDomNode* child, GError** error);
	GXmlDomNode* (*clone_node) (GXmlDomNode* self, gboolean deep, GError** error);
	GXmlDomNodeNodeType (*get_node_type) (GXmlDomNode* self);
	gchar* (*get_node_name) (GXmlDomNode* self);
	const gchar* (*get_base_uri) (GXmlDomNode* self);
	GXmlDomDocument* (*get_owner_document) (GXmlDomNode* self);
	void (*set_owner_document) (GXmlDomNode* self, GXmlDomDocument* value);
	GXmlDomNode* (*get_parent_node) (GXmlDomNode* self);
	GXmlDomElement* (*get_parent_element) (GXmlDomNode* self);
	GXmlDomNodeList* (*get_child_nodes) (GXmlDomNode* self);
	GXmlDomNode* (*get_first_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_last_child) (GXmlDomNode* self);
	GXmlDomNode* (*get_previous_sibling) (GXmlDomNode* self);
	GXmlDomNode* (*get_next_sibling) (GXmlDomNode* self);
	gchar* (*get_node_value) (GXmlDomNode* self);
	void (*set_node_value) (GXmlDomNode* self, const gchar* value);
	gchar* (*get_text_content) (GXmlDomNode* self);
	void (*set_text_content) (GXmlDomNode* self, const gchar* value);
};

struct _GXmlDomTokenListIface {
	GTypeInterface parent_iface;
	gchar* (*item) (GXmlDomTokenList* self, gint index);
	gboolean (*contains) (GXmlDomTokenList* self, const gchar* token, GError** error);
	void (*add) (GXmlDomTokenList* self, gchar** tokens, gint tokens_length1, GError** error);
	void (*remove) (GXmlDomTokenList* self, gchar** tokens, gint tokens_length1);
	gboolean (*toggle) (GXmlDomTokenList* self, const gchar* token, gboolean force, gboolean _auto_, GError** error);
	gchar* (*to_string) (GXmlDomTokenList* self);
	gint (*get_length) (GXmlDomTokenList* self);
};

struct _GXmlDomNamedNodeMapIface {
	GTypeInterface parent_iface;
	GXmlDomNode* (*item) (GXmlDomNamedNodeMap* self, gint index);
	GXmlDomNode* (*get_named_item) (GXmlDomNamedNodeMap* self, const gchar* name);
	GXmlDomNode* (*set_named_item) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	GXmlDomNode* (*remove_named_item) (GXmlDomNamedNodeMap* self, const gchar* name, GError** error);
	GXmlDomNode* (*remove_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* localName, GError** error);
	GXmlDomNode* (*get_named_item_ns) (GXmlDomNamedNodeMap* self, const gchar* namespace_uri, const gchar* local_name, GError** error);
	GXmlDomNode* (*set_named_item_ns) (GXmlDomNamedNodeMap* self, GXmlDomNode* node, GError** error);
	gint (*get_length) (GXmlDomNamedNodeMap* self);
};

struct _GXmlDomElementIface {
	GTypeInterface parent_iface;
	gchar* (*get_attribute) (GXmlDomElement* self, const gchar* name);
	gchar* (*get_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	void (*set_attribute) (GXmlDomElement* self, const gchar* name, const gchar* value, GError** error);
	void (*set_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* name, const gchar* value, GError** error);
	void (*remove_attribute) (GXmlDomElement* self, const gchar* name);
	void (*remove_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	gboolean (*has_attribute) (GXmlDomElement* self, const gchar* name);
	gboolean (*has_attribute_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name) (GXmlDomElement* self, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_tag_name_ns) (GXmlDomElement* self, const gchar* namespace, const gchar* local_name);
	GXmlDomHTMLCollection* (*get_elements_by_class_name) (GXmlDomElement* self, const gchar* class_names);
	gboolean (*matches) (GXmlDomElement* self, const gchar* selectors, GError** error);
	void (*read_from_uri) (GXmlDomElement* self, const gchar* uri, GError** error);
	void (*read_from_uri_async) (GXmlDomElement* self, const gchar* uri, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_uri_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	void (*read_from_file) (GXmlDomElement* self, GFile* f, GCancellable* cancellable, GError** error);
	void (*read_from_file_async) (GXmlDomElement* self, GFile* f, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_file_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	void (*read_from_stream) (GXmlDomElement* self, GInputStream* istream, GCancellable* cancellable, GError** error);
	void (*read_from_stream_async) (GXmlDomElement* self, GInputStream* istream, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_stream_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	void (*read_from_string) (GXmlDomElement* self, const gchar* str, GCancellable* cancellable, GError** error);
	void (*read_from_string_async) (GXmlDomElement* self, const gchar* str, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*read_from_string_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	gchar* (*write_string) (GXmlDomElement* self, GCancellable* cancellable, GError** error);
	void (*write_string_async) (GXmlDomElement* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar* (*write_string_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	void (*write_file) (GXmlDomElement* self, GFile* f, GCancellable* cancellable, GError** error);
	void (*write_file_async) (GXmlDomElement* self, GFile* f, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_file_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	void (*write_stream) (GXmlDomElement* self, GOutputStream* stream, GError** error);
	void (*write_stream_async) (GXmlDomElement* self, GOutputStream* stream, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*write_stream_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	GInputStream* (*create_stream) (GXmlDomElement* self, GError** error);
	void (*create_stream_async) (GXmlDomElement* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GInputStream* (*create_stream_finish) (GXmlDomElement* self, GAsyncResult* _res_, GError** error);
	gchar* (*get_namespace_uri) (GXmlDomElement* self);
	gchar* (*get_prefix) (GXmlDomElement* self);
	gchar* (*get_local_name) (GXmlDomElement* self);
	gchar* (*get_tag_name) (GXmlDomElement* self);
	gchar* (*get_id) (GXmlDomElement* self);
	void (*set_id) (GXmlDomElement* self, const gchar* value);
	gchar* (*get_class_name) (GXmlDomElement* self);
	void (*set_class_name) (GXmlDomElement* self, const gchar* value);
	GXmlDomTokenList* (*get_class_list) (GXmlDomElement* self);
	GXmlDomNamedNodeMap* (*get_attributes) (GXmlDomElement* self);
};

struct _GXmlCollectionIface {
	GTypeInterface parent_iface;
	void (*search) (GXmlCollection* self, GError** error);
	GXmlDomElement* (*get_item) (GXmlCollection* self, gint index, GError** error);
	void (*append) (GXmlCollection* self, GXmlDomElement* node, GError** error);
	void (*initialize) (GXmlCollection* self, GType t, GError** error);
	GXmlDomElement* (*create_item) (GXmlCollection* self);
	gboolean (*validate_append) (GXmlCollection* self, gint index, GXmlDomElement* element, GError** error);
	void (*clear) (GXmlCollection* self, GError** error);
	GQueue* (*get_nodes_index) (GXmlCollection* self);
	GXmlDomElement* (*get_element) (GXmlCollection* self);
	void (*set_element) (GXmlCollection* self, GXmlDomElement* value);
	const gchar* (*get_items_name) (GXmlCollection* self);
	GType (*get_items_type) (GXmlCollection* self);
	void (*set_items_type) (GXmlCollection* self, GType value);
	gint (*get_length) (GXmlCollection* self);
};

struct _GXmlBaseCollection {
	GObject parent_instance;
	GXmlBaseCollectionPrivate * priv;
	GQueue* _nodes_index;
	GXmlGomElement* _element;
	gchar* _items_name;
	GType _items_type;
};

struct _GXmlBaseCollectionClass {
	GObjectClass parent_class;
	gboolean (*validate_append) (GXmlBaseCollection* self, gint index, GXmlDomElement* element, GError** error);
	void (*clear) (GXmlBaseCollection* self, GError** error);
};

struct _GXmlThreeMapIface {
	GTypeInterface parent_iface;
	GXmlDomElement* (*item) (GXmlThreeMap* self, const gchar* primary_key, const gchar* secondary_key, const gchar* third_key);
	gboolean (*has_primary_key) (GXmlThreeMap* self, const gchar* key);
	gboolean (*has_secondary_key) (GXmlThreeMap* self, const gchar* pkey, const gchar* key);
	gboolean (*has_third_key) (GXmlThreeMap* self, const gchar* pkey, const gchar* skey, const gchar* key);
	GeeSet* (*secondary_keys_set) (GXmlThreeMap* self, const gchar* pkey);
	GeeSet* (*third_keys_set) (GXmlThreeMap* self, const gchar* pkey, const gchar* skey);
	const gchar* (*get_attribute_primary_key) (GXmlThreeMap* self);
	void (*set_attribute_primary_key) (GXmlThreeMap* self, const gchar* value);
	const gchar* (*get_attribute_secondary_key) (GXmlThreeMap* self);
	void (*set_attribute_secondary_key) (GXmlThreeMap* self, const gchar* value);
	const gchar* (*get_attribute_third_key) (GXmlThreeMap* self);
	void (*set_attribute_third_key) (GXmlThreeMap* self, const gchar* value);
	GeeSet* (*get_primary_keys_set) (GXmlThreeMap* self);
};

struct _GXmlGomHashThreeMap {
	GXmlBaseCollection parent_instance;
	GXmlGomHashThreeMapPrivate * priv;
	GeeHashMap* _hashtable;
	gchar* _attribute_primary_key;
	gchar* _attribute_secondary_key;
	gchar* _attribute_third_key;
};

struct _GXmlGomHashThreeMapClass {
	GXmlBaseCollectionClass parent_class;
};

struct _GXmlMappeableElementThreeKeyIface {
	GTypeInterface parent_iface;
	gchar* (*get_map_pkey) (GXmlMappeableElementThreeKey* self);
	gchar* (*get_map_skey) (GXmlMappeableElementThreeKey* self);
	gchar* (*get_map_tkey) (GXmlMappeableElementThreeKey* self);
};

static gpointer gxml_gom_hash_three_map_parent_class = NULL;
static GXmlThreeMapIface * gxml_gom_hash_three_map_gxml_three_map_parent_iface = NULL;

GType gxml_dom_time_stamp_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_phase_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_listener_get_type (void) G_GNUC_CONST;
GType gxml_dom_event_target_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_document_position_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_node_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_html_collection_get_type (void) G_GNUC_CONST;
GType gxml_dom_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_element_parent_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_fragment_get_type (void) G_GNUC_CONST;
GType gxml_dom_non_document_type_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_child_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_character_data_get_type (void) G_GNUC_CONST;
GType gxml_dom_text_get_type (void) G_GNUC_CONST;
GType gxml_dom_comment_get_type (void) G_GNUC_CONST;
GType gxml_dom_processing_instruction_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_boundary_points_get_type (void) G_GNUC_CONST;
GType gxml_dom_range_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_filter_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_iterator_get_type (void) G_GNUC_CONST;
GType gxml_dom_tree_walker_get_type (void) G_GNUC_CONST;
GType gxml_parser_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_type_get_type (void) G_GNUC_CONST;
GType gxml_dom_xml_document_get_type (void) G_GNUC_CONST;
GType gxml_element_list_get_type (void) G_GNUC_CONST;
GType gxml_namespace_get_type (void) G_GNUC_CONST;
GType gxml_node_type_get_type (void) G_GNUC_CONST;
GType gxml_node_get_type (void) G_GNUC_CONST;
GType gxml_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_implementation_get_type (void) G_GNUC_CONST;
GType gxml_dom_document_get_type (void) G_GNUC_CONST;
GType gxml_dom_node_get_type (void) G_GNUC_CONST;
GType gxml_dom_token_list_get_type (void) G_GNUC_CONST;
GType gxml_dom_named_node_map_get_type (void) G_GNUC_CONST;
GType gxml_dom_element_get_type (void) G_GNUC_CONST;
GType gxml_collection_get_type (void) G_GNUC_CONST;
GType gxml_base_collection_get_type (void) G_GNUC_CONST;
GType gxml_gom_node_get_type (void) G_GNUC_CONST;
GType gxml_gom_element_get_type (void) G_GNUC_CONST;
GType gxml_three_map_get_type (void) G_GNUC_CONST;
GType gxml_gom_hash_three_map_get_type (void) G_GNUC_CONST;
void gxml_gom_hash_three_map_initialize_element_with_keys (GXmlGomHashThreeMap* self,
                                                           GXmlGomElement* element,
                                                           GType items_type,
                                                           const gchar* attribute_primary_key,
                                                           const gchar* attribute_secondary_key,
                                                           const gchar* attribute_third_key,
                                                           GError** error);
void gxml_collection_initialize (GXmlCollection* self,
                                 GType t,
                                 GError** error);
void gxml_base_collection_initialize_element (GXmlBaseCollection* self,
                                              GXmlGomElement* e,
                                              GError** error);
void gxml_gom_hash_three_map_initialize_with_keys (GXmlGomHashThreeMap* self,
                                                   GType items_type,
                                                   const gchar* attribute_primary_key,
                                                   const gchar* attribute_secondary_key,
                                                   const gchar* attribute_third_key,
                                                   GError** error);
GXmlDomElement* gxml_gom_hash_three_map_get (GXmlGomHashThreeMap* self,
                                             const gchar* primary_key,
                                             const gchar* secondary_key,
                                             const gchar* third_key);
GXmlDomNodeList* gxml_dom_node_get_child_nodes (GXmlDomNode* self);
static gboolean gxml_gom_hash_three_map_real_has_primary_key (GXmlThreeMap* base,
                                                       const gchar* key);
static gboolean gxml_gom_hash_three_map_real_has_secondary_key (GXmlThreeMap* base,
                                                         const gchar* pkey,
                                                         const gchar* key);
static gboolean gxml_gom_hash_three_map_real_has_third_key (GXmlThreeMap* base,
                                                     const gchar* pkey,
                                                     const gchar* skey,
                                                     const gchar* key);
GList* gxml_gom_hash_three_map_get_primary_keys (GXmlGomHashThreeMap* self);
static void _g_free0_ (gpointer var);
static inline void _g_list_free__g_free0_ (GList* self);
GList* gxml_gom_hash_three_map_get_secondary_keys (GXmlGomHashThreeMap* self,
                                                   const gchar* pkey);
GList* gxml_gom_hash_three_map_get_third_keys (GXmlGomHashThreeMap* self,
                                               const gchar* pkey,
                                               const gchar* skey);
static gboolean gxml_gom_hash_three_map_real_validate_append (GXmlBaseCollection* base,
                                                       gint index,
                                                       GXmlDomElement* element,
                                                       GError** error);
const gchar* gxml_three_map_get_attribute_primary_key (GXmlThreeMap* self);
const gchar* gxml_three_map_get_attribute_secondary_key (GXmlThreeMap* self);
const gchar* gxml_three_map_get_attribute_third_key (GXmlThreeMap* self);
gchar* gxml_dom_element_get_attribute (GXmlDomElement* self,
                                       const gchar* name);
GType gxml_collection_get_items_type (GXmlCollection* self);
GType gxml_mappeable_element_three_key_get_type (void) G_GNUC_CONST;
gchar* gxml_mappeable_element_three_key_get_map_pkey (GXmlMappeableElementThreeKey* self);
gchar* gxml_mappeable_element_three_key_get_map_skey (GXmlMappeableElementThreeKey* self);
gchar* gxml_mappeable_element_three_key_get_map_tkey (GXmlMappeableElementThreeKey* self);
static void gxml_gom_hash_three_map_real_clear (GXmlBaseCollection* base,
                                         GError** error);
static GXmlDomElement* gxml_gom_hash_three_map_real_item (GXmlThreeMap* base,
                                                   const gchar* primary_key,
                                                   const gchar* secondary_key,
                                                   const gchar* third_key);
static GeeSet* gxml_gom_hash_three_map_real_secondary_keys_set (GXmlThreeMap* base,
                                                         const gchar* pkey);
static GeeSet* gxml_gom_hash_three_map_real_third_keys_set (GXmlThreeMap* base,
                                                     const gchar* pkey,
                                                     const gchar* skey);
GXmlGomHashThreeMap* gxml_gom_hash_three_map_new (void);
GXmlGomHashThreeMap* gxml_gom_hash_three_map_construct (GType object_type);
GXmlBaseCollection* gxml_base_collection_construct (GType object_type);
static void gxml_gom_hash_three_map_finalize (GObject * obj);
GeeSet* gxml_three_map_get_primary_keys_set (GXmlThreeMap* self);
static void _vala_gxml_gom_hash_three_map_get_property (GObject * object,
                                                 guint property_id,
                                                 GValue * value,
                                                 GParamSpec * pspec);
void gxml_three_map_set_attribute_primary_key (GXmlThreeMap* self,
                                               const gchar* value);
void gxml_three_map_set_attribute_secondary_key (GXmlThreeMap* self,
                                                 const gchar* value);
void gxml_three_map_set_attribute_third_key (GXmlThreeMap* self,
                                             const gchar* value);
static void _vala_gxml_gom_hash_three_map_set_property (GObject * object,
                                                 guint property_id,
                                                 const GValue * value,
                                                 GParamSpec * pspec);

/**
   * Convenient function to initialize a {@link GomHashMap} collection, using
   * given element, items' type and name.
   */
void
gxml_gom_hash_three_map_initialize_element_with_keys (GXmlGomHashThreeMap* self,
                                                      GXmlGomElement* element,
                                                      GType items_type,
                                                      const gchar* attribute_primary_key,
                                                      const gchar* attribute_secondary_key,
                                                      const gchar* attribute_third_key,
                                                      GError** error)
{
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (element != NULL);
	g_return_if_fail (attribute_primary_key != NULL);
	g_return_if_fail (attribute_secondary_key != NULL);
	g_return_if_fail (attribute_third_key != NULL);
	gxml_collection_initialize ((GXmlCollection*) self, items_type, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_propagate_error (error, _inner_error0_);
		return;
	}
	gxml_base_collection_initialize_element ((GXmlBaseCollection*) self, element, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_propagate_error (error, _inner_error0_);
		return;
	}
	_tmp0_ = g_strdup (attribute_primary_key);
	_g_free0 (self->_attribute_primary_key);
	self->_attribute_primary_key = _tmp0_;
	_tmp1_ = g_strdup (attribute_secondary_key);
	_g_free0 (self->_attribute_secondary_key);
	self->_attribute_secondary_key = _tmp1_;
	_tmp2_ = g_strdup (attribute_third_key);
	_g_free0 (self->_attribute_third_key);
	self->_attribute_third_key = _tmp2_;
}

/**
   * Convenient function to initialize a {@link GomHashMap} collection, using
   * given element, items' type and name.
   *
   * Using this method at construction time of derived classes.
   */
void
gxml_gom_hash_three_map_initialize_with_keys (GXmlGomHashThreeMap* self,
                                              GType items_type,
                                              const gchar* attribute_primary_key,
                                              const gchar* attribute_secondary_key,
                                              const gchar* attribute_third_key,
                                              GError** error)
{
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	GError* _inner_error0_ = NULL;
	g_return_if_fail (self != NULL);
	g_return_if_fail (attribute_primary_key != NULL);
	g_return_if_fail (attribute_secondary_key != NULL);
	g_return_if_fail (attribute_third_key != NULL);
	gxml_collection_initialize ((GXmlCollection*) self, items_type, &_inner_error0_);
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
		g_propagate_error (error, _inner_error0_);
		return;
	}
	_tmp0_ = g_strdup (attribute_primary_key);
	_g_free0 (self->_attribute_primary_key);
	self->_attribute_primary_key = _tmp0_;
	_tmp1_ = g_strdup (attribute_secondary_key);
	_g_free0 (self->_attribute_secondary_key);
	self->_attribute_secondary_key = _tmp1_;
	_tmp2_ = g_strdup (attribute_third_key);
	_g_free0 (self->_attribute_third_key);
	self->_attribute_third_key = _tmp2_;
}

/**
   * Returns an {@link DomElement} in the collection using given string keys.
   */
GXmlDomElement*
gxml_gom_hash_three_map_get (GXmlGomHashThreeMap* self,
                             const gchar* primary_key,
                             const gchar* secondary_key,
                             const gchar* third_key)
{
	GXmlDomElement* result = NULL;
	GeeHashMap* _tmp0_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	GeeHashMap* _tmp3_;
	GeeHashMap* _tmp4_;
	GeeHashMap* hte = NULL;
	GeeHashMap* _tmp5_;
	gpointer _tmp6_;
	GeeHashMap* _tmp7_;
	GeeHashMap* _tmp8_;
	gint i = 0;
	GeeHashMap* _tmp9_;
	gpointer _tmp10_;
	GXmlGomElement* _tmp11_;
	GXmlDomNodeList* _tmp12_;
	GXmlDomNodeList* _tmp13_;
	GXmlDomNodeList* _tmp14_;
	gpointer _tmp15_;
	GXmlDomNode* _tmp16_;
	GXmlDomElement* _tmp17_;
	GXmlDomElement* _tmp18_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (primary_key != NULL, NULL);
	g_return_val_if_fail (secondary_key != NULL, NULL);
	g_return_val_if_fail (third_key != NULL, NULL);
	_tmp0_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, primary_key)) {
		result = NULL;
		return result;
	}
	_tmp1_ = self->_hashtable;
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, primary_key);
	ht = (GeeHashMap*) _tmp2_;
	_tmp3_ = ht;
	if (_tmp3_ == NULL) {
		result = NULL;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp4_ = ht;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp4_, secondary_key)) {
		result = NULL;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp5_ = ht;
	_tmp6_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp5_, secondary_key);
	hte = (GeeHashMap*) _tmp6_;
	_tmp7_ = hte;
	if (_tmp7_ == NULL) {
		result = NULL;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	_tmp8_ = hte;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp8_, third_key)) {
		result = NULL;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	_tmp9_ = hte;
	_tmp10_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp9_, secondary_key);
	i = (gint) ((gintptr) _tmp10_);
	_tmp11_ = ((GXmlBaseCollection*) self)->_element;
	_tmp12_ = gxml_dom_node_get_child_nodes ((GXmlDomNode*) _tmp11_);
	_tmp13_ = _tmp12_;
	_tmp14_ = _tmp13_;
	_tmp15_ = gee_list_get ((GeeList*) _tmp14_, i);
	_tmp16_ = (GXmlDomNode*) _tmp15_;
	_tmp17_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp16_, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) _tmp16_) : NULL;
	if (_tmp17_ == NULL) {
		_g_object_unref0 (_tmp16_);
	}
	_tmp18_ = _tmp17_;
	_g_object_unref0 (_tmp14_);
	result = _tmp18_;
	_g_object_unref0 (hte);
	_g_object_unref0 (ht);
	return result;
}

/**
   * Returns true if @key is used in collection as primery key.
   */
static gboolean
gxml_gom_hash_three_map_real_has_primary_key (GXmlThreeMap* base,
                                              const gchar* key)
{
	GXmlGomHashThreeMap * self;
	gboolean result = FALSE;
	GeeHashMap* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (key != NULL, FALSE);
	_tmp0_ = self->_hashtable;
	if (gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, key)) {
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

/**
   * Returns true if @key is used in collection as secondary key
   * with @pkey as primary.
   */
static gboolean
gxml_gom_hash_three_map_real_has_secondary_key (GXmlThreeMap* base,
                                                const gchar* pkey,
                                                const gchar* key)
{
	GXmlGomHashThreeMap * self;
	gboolean result = FALSE;
	GeeHashMap* _tmp0_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	GeeHashMap* _tmp3_;
	GeeHashMap* _tmp4_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (pkey != NULL, FALSE);
	g_return_val_if_fail (key != NULL, FALSE);
	_tmp0_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, pkey)) {
		result = FALSE;
		return result;
	}
	_tmp1_ = self->_hashtable;
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, pkey);
	ht = (GeeHashMap*) _tmp2_;
	_tmp3_ = ht;
	if (_tmp3_ == NULL) {
		result = FALSE;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp4_ = ht;
	if (gee_abstract_map_has_key ((GeeAbstractMap*) _tmp4_, key)) {
		result = TRUE;
		_g_object_unref0 (ht);
		return result;
	}
	result = FALSE;
	_g_object_unref0 (ht);
	return result;
}

/**
   * Returns true if @key is used in collection as third key with secondary key
   * and pkey as primary.
   */
static gboolean
gxml_gom_hash_three_map_real_has_third_key (GXmlThreeMap* base,
                                            const gchar* pkey,
                                            const gchar* skey,
                                            const gchar* key)
{
	GXmlGomHashThreeMap * self;
	gboolean result = FALSE;
	GeeHashMap* _tmp0_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	GeeHashMap* _tmp3_;
	GeeHashMap* hte = NULL;
	GeeHashMap* _tmp4_;
	gpointer _tmp5_;
	GeeHashMap* _tmp6_;
	GeeHashMap* _tmp7_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (pkey != NULL, FALSE);
	g_return_val_if_fail (skey != NULL, FALSE);
	g_return_val_if_fail (key != NULL, FALSE);
	_tmp0_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, pkey)) {
		result = FALSE;
		return result;
	}
	_tmp1_ = self->_hashtable;
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, pkey);
	ht = (GeeHashMap*) _tmp2_;
	_tmp3_ = ht;
	if (_tmp3_ == NULL) {
		result = FALSE;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp4_ = ht;
	_tmp5_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp4_, skey);
	hte = (GeeHashMap*) _tmp5_;
	_tmp6_ = hte;
	if (_tmp6_ == NULL) {
		result = FALSE;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	_tmp7_ = hte;
	if (gee_abstract_map_has_key ((GeeAbstractMap*) _tmp7_, key)) {
		result = TRUE;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	result = FALSE;
	_g_object_unref0 (hte);
	_g_object_unref0 (ht);
	return result;
}

/**
   * Returns list of primary keys used in collection.
   */
static void
_g_free0_ (gpointer var)
{
	var = (g_free (var), NULL);
}

static inline void
_g_list_free__g_free0_ (GList* self)
{
	g_list_free_full (self, (GDestroyNotify) _g_free0_);
}

GList*
gxml_gom_hash_three_map_get_primary_keys (GXmlGomHashThreeMap* self)
{
	GList* result = NULL;
	GList* l = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	l = NULL;
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp0_;
		GeeSet* _tmp1_;
		GeeSet* _tmp2_;
		GeeSet* _tmp3_;
		GeeIterator* _tmp4_;
		GeeIterator* _tmp5_;
		_tmp0_ = self->_hashtable;
		_tmp1_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp0_);
		_tmp2_ = _tmp1_;
		_tmp3_ = _tmp2_;
		_tmp4_ = gee_iterable_iterator ((GeeIterable*) _tmp3_);
		_tmp5_ = _tmp4_;
		_g_object_unref0 (_tmp3_);
		_k_it = _tmp5_;
		while (TRUE) {
			GeeIterator* _tmp6_;
			gchar* k = NULL;
			GeeIterator* _tmp7_;
			gpointer _tmp8_;
			const gchar* _tmp9_;
			gchar* _tmp10_;
			_tmp6_ = _k_it;
			if (!gee_iterator_next (_tmp6_)) {
				break;
			}
			_tmp7_ = _k_it;
			_tmp8_ = gee_iterator_get (_tmp7_);
			k = (gchar*) _tmp8_;
			_tmp9_ = k;
			_tmp10_ = g_strdup (_tmp9_);
			l = g_list_append (l, _tmp10_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = l;
	return result;
}

/**
   * Returns list of secondary keys used in collection with pkey as primary key.
   */
GList*
gxml_gom_hash_three_map_get_secondary_keys (GXmlGomHashThreeMap* self,
                                            const gchar* pkey)
{
	GList* result = NULL;
	GList* l = NULL;
	GeeHashMap* _tmp0_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	GeeHashMap* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (pkey != NULL, NULL);
	l = NULL;
	_tmp0_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, pkey)) {
		result = l;
		return result;
	}
	_tmp1_ = self->_hashtable;
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, pkey);
	ht = (GeeHashMap*) _tmp2_;
	_tmp3_ = ht;
	if (_tmp3_ == NULL) {
		result = l;
		_g_object_unref0 (ht);
		return result;
	}
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp4_;
		GeeSet* _tmp5_;
		GeeSet* _tmp6_;
		GeeSet* _tmp7_;
		GeeIterator* _tmp8_;
		GeeIterator* _tmp9_;
		_tmp4_ = ht;
		_tmp5_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp4_);
		_tmp6_ = _tmp5_;
		_tmp7_ = _tmp6_;
		_tmp8_ = gee_iterable_iterator ((GeeIterable*) _tmp7_);
		_tmp9_ = _tmp8_;
		_g_object_unref0 (_tmp7_);
		_k_it = _tmp9_;
		while (TRUE) {
			GeeIterator* _tmp10_;
			gchar* k = NULL;
			GeeIterator* _tmp11_;
			gpointer _tmp12_;
			const gchar* _tmp13_;
			gchar* _tmp14_;
			_tmp10_ = _k_it;
			if (!gee_iterator_next (_tmp10_)) {
				break;
			}
			_tmp11_ = _k_it;
			_tmp12_ = gee_iterator_get (_tmp11_);
			k = (gchar*) _tmp12_;
			_tmp13_ = k;
			_tmp14_ = g_strdup (_tmp13_);
			l = g_list_append (l, _tmp14_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = l;
	_g_object_unref0 (ht);
	return result;
}

/**
   * Returns list of third keys used in collection with pkey as primary key
   * and skey as secondary key.
   */
GList*
gxml_gom_hash_three_map_get_third_keys (GXmlGomHashThreeMap* self,
                                        const gchar* pkey,
                                        const gchar* skey)
{
	GList* result = NULL;
	GList* l = NULL;
	GeeHashMap* _tmp0_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	GeeHashMap* _tmp3_;
	GeeHashMap* hte = NULL;
	GeeHashMap* _tmp4_;
	gpointer _tmp5_;
	GeeHashMap* _tmp6_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (pkey != NULL, NULL);
	g_return_val_if_fail (skey != NULL, NULL);
	l = NULL;
	_tmp0_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, pkey)) {
		result = l;
		return result;
	}
	_tmp1_ = self->_hashtable;
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, pkey);
	ht = (GeeHashMap*) _tmp2_;
	_tmp3_ = ht;
	if (_tmp3_ == NULL) {
		result = l;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp4_ = ht;
	_tmp5_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp4_, skey);
	hte = (GeeHashMap*) _tmp5_;
	_tmp6_ = hte;
	if (_tmp6_ == NULL) {
		result = l;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp7_;
		GeeSet* _tmp8_;
		GeeSet* _tmp9_;
		GeeSet* _tmp10_;
		GeeIterator* _tmp11_;
		GeeIterator* _tmp12_;
		_tmp7_ = hte;
		_tmp8_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp7_);
		_tmp9_ = _tmp8_;
		_tmp10_ = _tmp9_;
		_tmp11_ = gee_iterable_iterator ((GeeIterable*) _tmp10_);
		_tmp12_ = _tmp11_;
		_g_object_unref0 (_tmp10_);
		_k_it = _tmp12_;
		while (TRUE) {
			GeeIterator* _tmp13_;
			gchar* k = NULL;
			GeeIterator* _tmp14_;
			gpointer _tmp15_;
			const gchar* _tmp16_;
			gchar* _tmp17_;
			_tmp13_ = _k_it;
			if (!gee_iterator_next (_tmp13_)) {
				break;
			}
			_tmp14_ = _k_it;
			_tmp15_ = gee_iterator_get (_tmp14_);
			k = (gchar*) _tmp15_;
			_tmp16_ = k;
			_tmp17_ = g_strdup (_tmp16_);
			l = g_list_append (l, _tmp17_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = l;
	_g_object_unref0 (hte);
	_g_object_unref0 (ht);
	return result;
}

/**
   * Validates if given element has a {@link attribute_primary_key},
   * {@link attribute_secondary_key} and
   * {@link attribute_third_key} set,
   * if so adds a new keys pointing to given index and returns true.
   *
   * Attribute should be a valid {@link DomElement} attribute or
   * a {@link GomObject} property identified using a nick with a '::' prefix.
   *
   * If there are more elements with same keys, they are kept as child nodes
   * but the one in collection will be the last one to be found.
   *
   * Return: false if element should not be added to collection.
   */
static gboolean
gxml_gom_hash_three_map_real_validate_append (GXmlBaseCollection* base,
                                              gint index,
                                              GXmlDomElement* element,
                                              GError** error)
{
	GXmlGomHashThreeMap * self;
	gboolean result = FALSE;
	gchar* pkey = NULL;
	gchar* skey = NULL;
	gchar* tkey = NULL;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	gboolean _tmp42_ = FALSE;
	gboolean _tmp43_ = FALSE;
	const gchar* _tmp44_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp47_;
	const gchar* _tmp48_;
	gpointer _tmp49_;
	GeeHashMap* _tmp50_;
	GeeHashMap* hte = NULL;
	GeeHashMap* _tmp52_;
	const gchar* _tmp53_;
	gpointer _tmp54_;
	GeeHashMap* _tmp55_;
	GeeHashMap* _tmp57_;
	const gchar* _tmp58_;
	GeeHashMap* _tmp62_;
	const gchar* _tmp63_;
	GeeHashMap* _tmp67_;
	const gchar* _tmp68_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (element != NULL, FALSE);
	if (!G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_GOM_ELEMENT)) {
		result = FALSE;
		return result;
	}
	pkey = NULL;
	skey = NULL;
	tkey = NULL;
	_tmp2_ = gxml_three_map_get_attribute_primary_key ((GXmlThreeMap*) self);
	_tmp3_ = _tmp2_;
	if (_tmp3_ != NULL) {
		const gchar* _tmp4_;
		const gchar* _tmp5_;
		_tmp4_ = gxml_three_map_get_attribute_secondary_key ((GXmlThreeMap*) self);
		_tmp5_ = _tmp4_;
		_tmp1_ = _tmp5_ != NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		const gchar* _tmp6_;
		const gchar* _tmp7_;
		_tmp6_ = gxml_three_map_get_attribute_third_key ((GXmlThreeMap*) self);
		_tmp7_ = _tmp6_;
		_tmp0_ = _tmp7_ != NULL;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		const gchar* _tmp8_;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		const gchar* _tmp11_;
		const gchar* _tmp12_;
		gchar* _tmp13_;
		const gchar* _tmp14_;
		const gchar* _tmp15_;
		gchar* _tmp16_;
		gboolean _tmp17_ = FALSE;
		gboolean _tmp18_ = FALSE;
		const gchar* _tmp19_;
		_tmp8_ = gxml_three_map_get_attribute_primary_key ((GXmlThreeMap*) self);
		_tmp9_ = _tmp8_;
		_tmp10_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp9_);
		_g_free0 (pkey);
		pkey = _tmp10_;
		_tmp11_ = gxml_three_map_get_attribute_secondary_key ((GXmlThreeMap*) self);
		_tmp12_ = _tmp11_;
		_tmp13_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp12_);
		_g_free0 (skey);
		skey = _tmp13_;
		_tmp14_ = gxml_three_map_get_attribute_third_key ((GXmlThreeMap*) self);
		_tmp15_ = _tmp14_;
		_tmp16_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp15_);
		_g_free0 (tkey);
		tkey = _tmp16_;
		_tmp19_ = pkey;
		if (_tmp19_ == NULL) {
			_tmp18_ = TRUE;
		} else {
			const gchar* _tmp20_;
			_tmp20_ = skey;
			_tmp18_ = _tmp20_ == NULL;
		}
		if (_tmp18_) {
			_tmp17_ = TRUE;
		} else {
			const gchar* _tmp21_;
			_tmp21_ = tkey;
			_tmp17_ = _tmp21_ == NULL;
		}
		if (_tmp17_) {
			const gchar* _tmp22_;
			const gchar* _tmp23_;
			gchar* _tmp24_;
			gchar* _tmp25_;
			gchar* _tmp26_;
			const gchar* _tmp27_;
			const gchar* _tmp28_;
			gchar* _tmp29_;
			gchar* _tmp30_;
			gchar* _tmp31_;
			const gchar* _tmp32_;
			const gchar* _tmp33_;
			gchar* _tmp34_;
			gchar* _tmp35_;
			gchar* _tmp36_;
			_tmp22_ = gxml_three_map_get_attribute_primary_key ((GXmlThreeMap*) self);
			_tmp23_ = _tmp22_;
			_tmp24_ = g_utf8_strdown (_tmp23_, (gssize) -1);
			_tmp25_ = _tmp24_;
			_tmp26_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp25_);
			_g_free0 (pkey);
			pkey = _tmp26_;
			_g_free0 (_tmp25_);
			_tmp27_ = gxml_three_map_get_attribute_secondary_key ((GXmlThreeMap*) self);
			_tmp28_ = _tmp27_;
			_tmp29_ = g_utf8_strdown (_tmp28_, (gssize) -1);
			_tmp30_ = _tmp29_;
			_tmp31_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp30_);
			_g_free0 (skey);
			skey = _tmp31_;
			_g_free0 (_tmp30_);
			_tmp32_ = gxml_three_map_get_attribute_third_key ((GXmlThreeMap*) self);
			_tmp33_ = _tmp32_;
			_tmp34_ = g_utf8_strdown (_tmp33_, (gssize) -1);
			_tmp35_ = _tmp34_;
			_tmp36_ = gxml_dom_element_get_attribute (G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_DOM_ELEMENT) ? ((GXmlDomElement*) element) : NULL, _tmp35_);
			_g_free0 (tkey);
			tkey = _tmp36_;
			_g_free0 (_tmp35_);
		}
	} else {
		GType _tmp37_;
		GType _tmp38_;
		_tmp37_ = gxml_collection_get_items_type ((GXmlCollection*) self);
		_tmp38_ = _tmp37_;
		if (g_type_is_a (_tmp38_, GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY)) {
			gchar* _tmp39_;
			gchar* _tmp40_;
			gchar* _tmp41_;
			if (!G_TYPE_CHECK_INSTANCE_TYPE (element, GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY)) {
				result = FALSE;
				_g_free0 (tkey);
				_g_free0 (skey);
				_g_free0 (pkey);
				return result;
			}
			_tmp39_ = gxml_mappeable_element_three_key_get_map_pkey (G_TYPE_CHECK_INSTANCE_CAST (element, GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY, GXmlMappeableElementThreeKey));
			_g_free0 (pkey);
			pkey = _tmp39_;
			_tmp40_ = gxml_mappeable_element_three_key_get_map_skey (G_TYPE_CHECK_INSTANCE_CAST (element, GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY, GXmlMappeableElementThreeKey));
			_g_free0 (skey);
			skey = _tmp40_;
			_tmp41_ = gxml_mappeable_element_three_key_get_map_tkey (G_TYPE_CHECK_INSTANCE_CAST (element, GXML_TYPE_MAPPEABLE_ELEMENT_THREE_KEY, GXmlMappeableElementThreeKey));
			_g_free0 (tkey);
			tkey = _tmp41_;
		}
	}
	_tmp44_ = pkey;
	if (_tmp44_ == NULL) {
		_tmp43_ = TRUE;
	} else {
		const gchar* _tmp45_;
		_tmp45_ = skey;
		_tmp43_ = _tmp45_ == NULL;
	}
	if (_tmp43_) {
		_tmp42_ = TRUE;
	} else {
		const gchar* _tmp46_;
		_tmp46_ = tkey;
		_tmp42_ = _tmp46_ == NULL;
	}
	if (_tmp42_) {
		result = FALSE;
		_g_free0 (tkey);
		_g_free0 (skey);
		_g_free0 (pkey);
		return result;
	}
	_tmp47_ = self->_hashtable;
	_tmp48_ = pkey;
	_tmp49_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp47_, _tmp48_);
	ht = (GeeHashMap*) _tmp49_;
	_tmp50_ = ht;
	if (_tmp50_ == NULL) {
		GeeHashMap* _tmp51_;
		_tmp51_ = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, GEE_TYPE_HASH_MAP, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		_g_object_unref0 (ht);
		ht = _tmp51_;
	}
	_tmp52_ = ht;
	_tmp53_ = skey;
	_tmp54_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp52_, _tmp53_);
	hte = (GeeHashMap*) _tmp54_;
	_tmp55_ = hte;
	if (_tmp55_ == NULL) {
		GeeHashMap* _tmp56_;
		_tmp56_ = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, G_TYPE_INT, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		_g_object_unref0 (hte);
		hte = _tmp56_;
	}
	_tmp57_ = self->_hashtable;
	_tmp58_ = pkey;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp57_, _tmp58_)) {
		GeeHashMap* _tmp59_;
		const gchar* _tmp60_;
		GeeHashMap* _tmp61_;
		_tmp59_ = self->_hashtable;
		_tmp60_ = pkey;
		_tmp61_ = ht;
		gee_abstract_map_set ((GeeAbstractMap*) _tmp59_, _tmp60_, _tmp61_);
	}
	_tmp62_ = ht;
	_tmp63_ = skey;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp62_, _tmp63_)) {
		GeeHashMap* _tmp64_;
		const gchar* _tmp65_;
		GeeHashMap* _tmp66_;
		_tmp64_ = ht;
		_tmp65_ = skey;
		_tmp66_ = hte;
		gee_abstract_map_set ((GeeAbstractMap*) _tmp64_, _tmp65_, _tmp66_);
	}
	_tmp67_ = hte;
	_tmp68_ = tkey;
	gee_abstract_map_set ((GeeAbstractMap*) _tmp67_, _tmp68_, (gpointer) ((gintptr) index));
	result = TRUE;
	_g_object_unref0 (hte);
	_g_object_unref0 (ht);
	_g_free0 (tkey);
	_g_free0 (skey);
	_g_free0 (pkey);
	return result;
}

static void
gxml_gom_hash_three_map_real_clear (GXmlBaseCollection* base,
                                    GError** error)
{
	GXmlGomHashThreeMap * self;
	GeeHashMap* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, GEE_TYPE_HASH_MAP, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	_g_object_unref0 (self->_hashtable);
	self->_hashtable = _tmp0_;
}

static GXmlDomElement*
gxml_gom_hash_three_map_real_item (GXmlThreeMap* base,
                                   const gchar* primary_key,
                                   const gchar* secondary_key,
                                   const gchar* third_key)
{
	GXmlGomHashThreeMap * self;
	GXmlDomElement* result = NULL;
	GXmlDomElement* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (primary_key != NULL, NULL);
	g_return_val_if_fail (secondary_key != NULL, NULL);
	g_return_val_if_fail (third_key != NULL, NULL);
	_tmp0_ = gxml_gom_hash_three_map_get (self, primary_key, secondary_key, third_key);
	result = _tmp0_;
	return result;
}

static GeeSet*
gxml_gom_hash_three_map_real_secondary_keys_set (GXmlThreeMap* base,
                                                 const gchar* pkey)
{
	GXmlGomHashThreeMap * self;
	GeeSet* result = NULL;
	GeeHashSet* l = NULL;
	GeeHashSet* _tmp0_;
	GeeHashMap* _tmp1_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp2_;
	gpointer _tmp3_;
	GeeHashMap* _tmp4_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (pkey != NULL, NULL);
	_tmp0_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL, NULL, NULL, NULL);
	l = _tmp0_;
	_tmp1_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp1_, pkey)) {
		result = (GeeSet*) l;
		return result;
	}
	_tmp2_ = self->_hashtable;
	_tmp3_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp2_, pkey);
	ht = (GeeHashMap*) _tmp3_;
	_tmp4_ = ht;
	if (_tmp4_ == NULL) {
		result = (GeeSet*) l;
		_g_object_unref0 (ht);
		return result;
	}
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp5_;
		GeeSet* _tmp6_;
		GeeSet* _tmp7_;
		GeeSet* _tmp8_;
		GeeIterator* _tmp9_;
		GeeIterator* _tmp10_;
		_tmp5_ = ht;
		_tmp6_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp5_);
		_tmp7_ = _tmp6_;
		_tmp8_ = _tmp7_;
		_tmp9_ = gee_iterable_iterator ((GeeIterable*) _tmp8_);
		_tmp10_ = _tmp9_;
		_g_object_unref0 (_tmp8_);
		_k_it = _tmp10_;
		while (TRUE) {
			GeeIterator* _tmp11_;
			gchar* k = NULL;
			GeeIterator* _tmp12_;
			gpointer _tmp13_;
			GeeHashSet* _tmp14_;
			const gchar* _tmp15_;
			_tmp11_ = _k_it;
			if (!gee_iterator_next (_tmp11_)) {
				break;
			}
			_tmp12_ = _k_it;
			_tmp13_ = gee_iterator_get (_tmp12_);
			k = (gchar*) _tmp13_;
			_tmp14_ = l;
			_tmp15_ = k;
			gee_abstract_collection_add ((GeeAbstractCollection*) _tmp14_, _tmp15_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = (GeeSet*) l;
	_g_object_unref0 (ht);
	return result;
}

static GeeSet*
gxml_gom_hash_three_map_real_third_keys_set (GXmlThreeMap* base,
                                             const gchar* pkey,
                                             const gchar* skey)
{
	GXmlGomHashThreeMap * self;
	GeeSet* result = NULL;
	GeeHashSet* l = NULL;
	GeeHashSet* _tmp0_;
	GeeHashMap* _tmp1_;
	GeeHashMap* ht = NULL;
	GeeHashMap* _tmp2_;
	gpointer _tmp3_;
	GeeHashMap* _tmp4_;
	GeeHashMap* hte = NULL;
	GeeHashMap* _tmp5_;
	gpointer _tmp6_;
	GeeHashMap* _tmp7_;
	self = (GXmlGomHashThreeMap*) base;
	g_return_val_if_fail (pkey != NULL, NULL);
	g_return_val_if_fail (skey != NULL, NULL);
	_tmp0_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL, NULL, NULL, NULL);
	l = _tmp0_;
	_tmp1_ = self->_hashtable;
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp1_, pkey)) {
		result = (GeeSet*) l;
		return result;
	}
	_tmp2_ = self->_hashtable;
	_tmp3_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp2_, pkey);
	ht = (GeeHashMap*) _tmp3_;
	_tmp4_ = ht;
	if (_tmp4_ == NULL) {
		result = (GeeSet*) l;
		_g_object_unref0 (ht);
		return result;
	}
	_tmp5_ = ht;
	_tmp6_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp5_, skey);
	hte = (GeeHashMap*) _tmp6_;
	_tmp7_ = hte;
	if (_tmp7_ == NULL) {
		result = (GeeSet*) l;
		_g_object_unref0 (hte);
		_g_object_unref0 (ht);
		return result;
	}
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp8_;
		GeeSet* _tmp9_;
		GeeSet* _tmp10_;
		GeeSet* _tmp11_;
		GeeIterator* _tmp12_;
		GeeIterator* _tmp13_;
		_tmp8_ = hte;
		_tmp9_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp8_);
		_tmp10_ = _tmp9_;
		_tmp11_ = _tmp10_;
		_tmp12_ = gee_iterable_iterator ((GeeIterable*) _tmp11_);
		_tmp13_ = _tmp12_;
		_g_object_unref0 (_tmp11_);
		_k_it = _tmp13_;
		while (TRUE) {
			GeeIterator* _tmp14_;
			gchar* k = NULL;
			GeeIterator* _tmp15_;
			gpointer _tmp16_;
			GeeHashSet* _tmp17_;
			const gchar* _tmp18_;
			_tmp14_ = _k_it;
			if (!gee_iterator_next (_tmp14_)) {
				break;
			}
			_tmp15_ = _k_it;
			_tmp16_ = gee_iterator_get (_tmp15_);
			k = (gchar*) _tmp16_;
			_tmp17_ = l;
			_tmp18_ = k;
			gee_abstract_collection_add ((GeeAbstractCollection*) _tmp17_, _tmp18_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = (GeeSet*) l;
	_g_object_unref0 (hte);
	_g_object_unref0 (ht);
	return result;
}

GXmlGomHashThreeMap*
gxml_gom_hash_three_map_construct (GType object_type)
{
	GXmlGomHashThreeMap * self = NULL;
	self = (GXmlGomHashThreeMap*) gxml_base_collection_construct (object_type);
	return self;
}

GXmlGomHashThreeMap*
gxml_gom_hash_three_map_new (void)
{
	return gxml_gom_hash_three_map_construct (GXML_TYPE_GOM_HASH_THREE_MAP);
}

static const gchar*
gxml_gom_hash_three_map_real_get_attribute_primary_key (GXmlThreeMap* base)
{
	const gchar* result;
	GXmlGomHashThreeMap* self;
	const gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = self->_attribute_primary_key;
	result = _tmp0_;
	return result;
}

static void
gxml_gom_hash_three_map_real_set_attribute_primary_key (GXmlThreeMap* base,
                                                        const gchar* value)
{
	GXmlGomHashThreeMap* self;
	gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = g_strdup (value);
	_g_free0 (self->_attribute_primary_key);
	self->_attribute_primary_key = _tmp0_;
	g_object_notify_by_pspec ((GObject *) self, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY]);
}

static const gchar*
gxml_gom_hash_three_map_real_get_attribute_secondary_key (GXmlThreeMap* base)
{
	const gchar* result;
	GXmlGomHashThreeMap* self;
	const gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = self->_attribute_secondary_key;
	result = _tmp0_;
	return result;
}

static void
gxml_gom_hash_three_map_real_set_attribute_secondary_key (GXmlThreeMap* base,
                                                          const gchar* value)
{
	GXmlGomHashThreeMap* self;
	gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = g_strdup (value);
	_g_free0 (self->_attribute_secondary_key);
	self->_attribute_secondary_key = _tmp0_;
	g_object_notify_by_pspec ((GObject *) self, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY]);
}

static const gchar*
gxml_gom_hash_three_map_real_get_attribute_third_key (GXmlThreeMap* base)
{
	const gchar* result;
	GXmlGomHashThreeMap* self;
	const gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = self->_attribute_third_key;
	result = _tmp0_;
	return result;
}

static void
gxml_gom_hash_three_map_real_set_attribute_third_key (GXmlThreeMap* base,
                                                      const gchar* value)
{
	GXmlGomHashThreeMap* self;
	gchar* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = g_strdup (value);
	_g_free0 (self->_attribute_third_key);
	self->_attribute_third_key = _tmp0_;
	g_object_notify_by_pspec ((GObject *) self, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY]);
}

static GeeSet*
gxml_gom_hash_three_map_real_get_primary_keys_set (GXmlThreeMap* base)
{
	GeeSet* result;
	GXmlGomHashThreeMap* self;
	GeeHashSet* l = NULL;
	GeeHashSet* _tmp0_;
	self = (GXmlGomHashThreeMap*) base;
	_tmp0_ = gee_hash_set_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL, NULL, NULL, NULL);
	l = _tmp0_;
	{
		GeeIterator* _k_it = NULL;
		GeeHashMap* _tmp1_;
		GeeSet* _tmp2_;
		GeeSet* _tmp3_;
		GeeSet* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _tmp6_;
		_tmp1_ = self->_hashtable;
		_tmp2_ = gee_abstract_map_get_keys ((GeeAbstractMap*) _tmp1_);
		_tmp3_ = _tmp2_;
		_tmp4_ = _tmp3_;
		_tmp5_ = gee_iterable_iterator ((GeeIterable*) _tmp4_);
		_tmp6_ = _tmp5_;
		_g_object_unref0 (_tmp4_);
		_k_it = _tmp6_;
		while (TRUE) {
			GeeIterator* _tmp7_;
			gchar* k = NULL;
			GeeIterator* _tmp8_;
			gpointer _tmp9_;
			GeeHashSet* _tmp10_;
			const gchar* _tmp11_;
			_tmp7_ = _k_it;
			if (!gee_iterator_next (_tmp7_)) {
				break;
			}
			_tmp8_ = _k_it;
			_tmp9_ = gee_iterator_get (_tmp8_);
			k = (gchar*) _tmp9_;
			_tmp10_ = l;
			_tmp11_ = k;
			gee_abstract_collection_add ((GeeAbstractCollection*) _tmp10_, _tmp11_);
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	result = (GeeSet*) l;
	return result;
}

static void
gxml_gom_hash_three_map_class_init (GXmlGomHashThreeMapClass * klass,
                                    gpointer klass_data)
{
	gxml_gom_hash_three_map_parent_class = g_type_class_peek_parent (klass);
	((GXmlBaseCollectionClass *) klass)->validate_append = (gboolean (*) (GXmlBaseCollection*, gint, GXmlDomElement*, GError**)) gxml_gom_hash_three_map_real_validate_append;
	((GXmlBaseCollectionClass *) klass)->clear = (void (*) (GXmlBaseCollection*, GError**)) gxml_gom_hash_three_map_real_clear;
	G_OBJECT_CLASS (klass)->get_property = _vala_gxml_gom_hash_three_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gxml_gom_hash_three_map_set_property;
	G_OBJECT_CLASS (klass)->finalize = gxml_gom_hash_three_map_finalize;
	/**
	   * An attribute's name in items to be added and used to retrieve elements
	   * as primary key.
	   */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY] = g_param_spec_string ("attribute-primary-key", "attribute-primary-key", "attribute-primary-key", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	/**
	   * An attribute's name in items to be added and used to retrieve elements
	   * as secondary key.
	   */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY] = g_param_spec_string ("attribute-secondary-key", "attribute-secondary-key", "attribute-secondary-key", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	/**
	   * An attribute's name in items to be added and used to retrieve elements
	   * as third key.
	   */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY] = g_param_spec_string ("attribute-third-key", "attribute-third-key", "attribute-third-key", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GXML_GOM_HASH_THREE_MAP_PRIMARY_KEYS_SET_PROPERTY, gxml_gom_hash_three_map_properties[GXML_GOM_HASH_THREE_MAP_PRIMARY_KEYS_SET_PROPERTY] = g_param_spec_object ("primary-keys-set", "primary-keys-set", "primary-keys-set", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gxml_gom_hash_three_map_gxml_three_map_interface_init (GXmlThreeMapIface * iface,
                                                       gpointer iface_data)
{
	gxml_gom_hash_three_map_gxml_three_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->has_primary_key = (gboolean (*) (GXmlThreeMap*, const gchar*)) gxml_gom_hash_three_map_real_has_primary_key;
	iface->has_secondary_key = (gboolean (*) (GXmlThreeMap*, const gchar*, const gchar*)) gxml_gom_hash_three_map_real_has_secondary_key;
	iface->has_third_key = (gboolean (*) (GXmlThreeMap*, const gchar*, const gchar*, const gchar*)) gxml_gom_hash_three_map_real_has_third_key;
	iface->item = (GXmlDomElement* (*) (GXmlThreeMap*, const gchar*, const gchar*, const gchar*)) gxml_gom_hash_three_map_real_item;
	iface->secondary_keys_set = (GeeSet* (*) (GXmlThreeMap*, const gchar*)) gxml_gom_hash_three_map_real_secondary_keys_set;
	iface->third_keys_set = (GeeSet* (*) (GXmlThreeMap*, const gchar*, const gchar*)) gxml_gom_hash_three_map_real_third_keys_set;
	iface->get_attribute_primary_key = gxml_gom_hash_three_map_real_get_attribute_primary_key;
	iface->set_attribute_primary_key = gxml_gom_hash_three_map_real_set_attribute_primary_key;
	iface->get_attribute_secondary_key = gxml_gom_hash_three_map_real_get_attribute_secondary_key;
	iface->set_attribute_secondary_key = gxml_gom_hash_three_map_real_set_attribute_secondary_key;
	iface->get_attribute_third_key = gxml_gom_hash_three_map_real_get_attribute_third_key;
	iface->set_attribute_third_key = gxml_gom_hash_three_map_real_set_attribute_third_key;
	iface->get_primary_keys_set = gxml_gom_hash_three_map_real_get_primary_keys_set;
}

static void
gxml_gom_hash_three_map_instance_init (GXmlGomHashThreeMap * self,
                                       gpointer klass)
{
	GeeHashMap* _tmp0_;
	_tmp0_ = gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, GEE_TYPE_HASH_MAP, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	self->_hashtable = _tmp0_;
}

static void
gxml_gom_hash_three_map_finalize (GObject * obj)
{
	GXmlGomHashThreeMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMap);
	_g_object_unref0 (self->_hashtable);
	_g_free0 (self->_attribute_primary_key);
	_g_free0 (self->_attribute_secondary_key);
	_g_free0 (self->_attribute_third_key);
	G_OBJECT_CLASS (gxml_gom_hash_three_map_parent_class)->finalize (obj);
}

/**
 * A class impementing {@link Collection} to store references to
 * child {@link DomElement} of {@link Collection.element}, using three attributes in
 * items as primary, secondary tertiary keys or {@link MappeableElementThreeKey.get_map_pkey},
 * {@link MappeableElementThreeKey.get_map_skey}
 * and {@link MappeableElementThreeKey.get_map_tkey}
 * methods if {@link MappeableElementThreeKey} are implemented
 * by items to be added. All keys should be defined in node, otherwise
 * it is not added; but keeps it as a child node of actual
 * {@link Collection.element}.
 *
 * If {@link GomElement} to be added is of type {@link Collection.items_type}
 * and implements {@link MappeableElementThreeKey}, you should set
 * {@link attribute_primary_key}, {@link attribute_secondary_key}
 * and  {@link attribute_third_key}
 * to null in order to use returned value of {@link MappeableElementThreeKey.get_map_pkey},
 * {@link MappeableElementThreeKey.get_map_skey}
 * and {@link MappeableElementThreeKey.get_map_tkey}
 * as keys.
 *
 * {{{
 *   public class YourObject : GomElement, MappeableElementThirdKey {
 *    [Description (nick="::Name")]
 *    public string name { get; set; }
 *    public string code { get; set; }
 *    public string category { get; set; }
 *    public string get_map_primary_key () { return code; }
 *    public string get_map_secondary_key () { return name; }
 *    public string get_map_third_key () { return category; }
 *   }
 *   public class YourList : GomHashPairedMap {
 *    construct {
 *      try { initialize_with (typeof (YourObject)); }
 *      catch (GLib.Error e) {
 *        warning ("Initialization error for collection type: %s : %s"
 *             .printf (get_type ().name(), e.message));
 *      }
 *    }
 *   }
 * }}}
 */
GType
gxml_gom_hash_three_map_get_type (void)
{
	static volatile gsize gxml_gom_hash_three_map_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_gom_hash_three_map_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlGomHashThreeMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_gom_hash_three_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlGomHashThreeMap), 0, (GInstanceInitFunc) gxml_gom_hash_three_map_instance_init, NULL };
		static const GInterfaceInfo gxml_three_map_info = { (GInterfaceInitFunc) gxml_gom_hash_three_map_gxml_three_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_gom_hash_three_map_type_id;
		gxml_gom_hash_three_map_type_id = g_type_register_static (GXML_TYPE_BASE_COLLECTION, "GXmlGomHashThreeMap", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_gom_hash_three_map_type_id, GXML_TYPE_THREE_MAP, &gxml_three_map_info);
		g_once_init_leave (&gxml_gom_hash_three_map_type_id__volatile, gxml_gom_hash_three_map_type_id);
	}
	return gxml_gom_hash_three_map_type_id__volatile;
}

static void
_vala_gxml_gom_hash_three_map_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec)
{
	GXmlGomHashThreeMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMap);
	switch (property_id) {
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY:
		g_value_set_string (value, gxml_three_map_get_attribute_primary_key ((GXmlThreeMap*) self));
		break;
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY:
		g_value_set_string (value, gxml_three_map_get_attribute_secondary_key ((GXmlThreeMap*) self));
		break;
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY:
		g_value_set_string (value, gxml_three_map_get_attribute_third_key ((GXmlThreeMap*) self));
		break;
		case GXML_GOM_HASH_THREE_MAP_PRIMARY_KEYS_SET_PROPERTY:
		g_value_take_object (value, gxml_three_map_get_primary_keys_set ((GXmlThreeMap*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gxml_gom_hash_three_map_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec)
{
	GXmlGomHashThreeMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GXML_TYPE_GOM_HASH_THREE_MAP, GXmlGomHashThreeMap);
	switch (property_id) {
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_PRIMARY_KEY_PROPERTY:
		gxml_three_map_set_attribute_primary_key ((GXmlThreeMap*) self, g_value_get_string (value));
		break;
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_SECONDARY_KEY_PROPERTY:
		gxml_three_map_set_attribute_secondary_key ((GXmlThreeMap*) self, g_value_get_string (value));
		break;
		case GXML_GOM_HASH_THREE_MAP_ATTRIBUTE_THIRD_KEY_PROPERTY:
		gxml_three_map_set_attribute_third_key ((GXmlThreeMap*) self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

